@@ -39,7 +39,7 @@ else:
         fcntl.fcntl(fd, fcntl.F_SETFD, flags | fcntl.FD_CLOEXEC)
 
 
-__version__ = '0.7.2'
+__version__ = '0.7.3'
 apilevel = '2.0'
 threadsafety = 1
 paramstyle = 'qmark'
@@ -89,22 +89,6 @@ NUMBER = DBAPITypeObject(int, decimal.Decimal)
 DATETIME = DBAPITypeObject(datetime.datetime, datetime.date, datetime.time)
 ROWID = DBAPITypeObject()
 
-def convert_date(v):  # Convert datetime.date to BLR format data
-    i = v.month + 9
-    jy = v.year + (i // 12) -1
-    jm = i % 12
-    c = jy // 100
-    jy -= 100 * c
-    j = (146097*c) // 4 + (1461*jy) // 4 + (153*jm+2) // 5 + v.day - 678882
-    return bint_to_bytes(j, 4)
-
-def convert_time(v):  # Convert datetime.time to BLR format time
-    t = (v.hour*3600 + v.minute*60 + v.second) *10000 + v.microsecond // 100
-    return bint_to_bytes(t, 4)
-
-def convert_timestamp(v):   # Convert datetime.datetime to BLR format timestamp
-    return convert_date(v.date()) + convert_time(v.time())
-
 #------------------------------------------------------------------------------
 class XSQLVAR:
     type_length = {
@@ -412,6 +396,12 @@ class Cursor:
         self.stmt_handle = h
         self.arraysize = 1
 
+    def __enter__(self):
+        return self
+
+    def __exit__(self, exc, value, traceback):
+        self.close()
+
     @property
     def transaction(self):
         return self._transaction
@@ -461,7 +451,7 @@ class Cursor:
         except OperationalError:
             e = sys.exc_info()[1]
             if 335544665 in e.gds_codes:
-                raise IntegrityError(e.message, e.gds_codes, e.sql_code)
+                raise IntegrityError(e._message, e.gds_codes, e.sql_code)
         return stmt_type, stmt_handle
 
     def _callproc(self, query, params):
@@ -632,14 +622,40 @@ class Cursor:
 
     @property
     def description(self):
+        if not hasattr(self, "_xsqlda"):
+            return None
         return [(x.aliasname, x.sqltype, x.display_length(), x.io_length(),
                     x.precision(), x.sqlscale, True if x.null_ok else False)
                 for x in self._xsqlda]
 
     @property
     def rowcount(self):
-        # to be implemented
-        return -1
+        self.transaction.connection._op_info_sql(self.stmt_handle,
+                                     bytes([isc_info_sql_stmt_type]))
+        (h, oid, buf) = self.transaction.connection._op_response()
+        assert buf[:3] == bytes([0x15,0x04,0x00]) # isc_info_sql_stmt_type
+        stmt_type = bytes_to_int(buf[3:7])
+
+        self.transaction.connection._op_info_sql(self.stmt_handle,
+                                     bytes([isc_info_sql_records]))
+        (h, oid, buf) = self.transaction.connection._op_response()
+        assert buf[:3] == bytes([0x17,0x1d,0x00]) # isc_info_sql_records
+        if stmt_type == isc_info_sql_stmt_select:
+            # rowcount for select stmt changes based on num rows fetched
+            count = -1
+        elif stmt_type == isc_info_sql_stmt_insert:
+            assert buf[24:27] == bytes([0x0e,0x04,0x00])
+                                              # isc_info_req_insert_count
+            count = bytes_to_int(buf[27:31])
+        elif stmt_type == isc_info_sql_stmt_update:
+            assert buf[3:6] == bytes([0x0f,0x04,0x00])
+                                              # isc_info_req_update_count
+            count = bytes_to_int(buf[6:10])
+        elif stmt_type == isc_info_sql_stmt_delete:
+            assert buf[10:13] == bytes([0x10,0x04,0x00])
+                                              # isc_info_req_delete_count
+            count = bytes_to_int(buf[13:17])
+        return count
 
 class EventConduit(WireProtocol):
     def __init__(self, conn, names):
@@ -690,7 +706,7 @@ class Connection(WireProtocol):
             user = os.environ['USERNAME']
             hostname = os.environ['COMPUTERNAME']
         else:
-            user = os.environ['USER']
+            user = os.environ.get('USER', '')
             hostname = socket.gethostname()
         return bytes([1] + [len(user)] + [ord(c) for c in user]
                 + [4] + [len(hostname)] + [ord(c) for c in hostname] + [6, 0])
@@ -778,6 +794,15 @@ class Connection(WireProtocol):
         self.closed = False
         self.last_event_id = 0
 
+    def __enter__(self):
+        return self
+
+    def __exit__(self, exc, value, traceback):
+        if exc:
+            self.rollback()
+        else:
+            self.commit()
+
     def set_isolation_level(self, isolation_level):
         self.isolation_level = isolation_level
 
