@@ -31,12 +31,28 @@ INFO_SQL_SELECT_DESCRIBE_VARS = bytes([
     isc_info_sql_alias,
     isc_info_sql_describe_end])
 
+def convert_date(v):  # Convert datetime.date to BLR format data
+    i = v.month + 9
+    jy = v.year + (i // 12) -1
+    jm = i % 12
+    c = jy // 100
+    jy -= 100 * c
+    j = (146097*c) // 4 + (1461*jy) // 4 + (153*jm+2) // 5 + v.day - 678882
+    return bint_to_bytes(j, 4)
+
+def convert_time(v):  # Convert datetime.time to BLR format time
+    t = (v.hour*3600 + v.minute*60 + v.second) *10000 + v.microsecond // 100
+    return bint_to_bytes(t, 4)
+
+def convert_timestamp(v):   # Convert datetime.datetime to BLR format timestamp
+    return convert_date(v.date()) + convert_time(v.time())
+
 def wire_operation(fn):
     if not DEBUG:
         return fn
-    def f(*args):
+    def f(*args, **kwargs):
         print('<----', fn, args)
-        r = fn(*args)
+        r = fn(*args, **kwargs)
         print(fn, '---->')
         return r
     return f
@@ -131,7 +147,7 @@ def params_to_blr(params):
             blr += bytes([14, nbytes & 255, nbytes >> 8])
         elif t == int:
             v = bint_to_bytes(p, 4)
-            blr += bytes([7, 0])
+            blr += bytes([8, 0])    # blr_long
         elif t == decimal.Decimal or t == float:
             if t == float:
                 p = decimal.Decimal(str(p))
